import json

from httpx import AsyncClient, Request
from allauth.socialaccount.models import SocialToken
from allauth.socialaccount.providers.gitlab.views import GitLabOAuth2Adapter

GITLAB_BASE_URL = f"{GitLabOAuth2Adapter.provider_base_url}/api/v4/"


async def proxy(path, user, query_string, body, method):
    social_token = SocialToken.objects.get(
        account__user=user, account__provider="gitlab"
    )
    headers = get_headers(social_token.token)
    url = f"{GITLAB_BASE_URL}{path}"
    if query_string:
        url += "?" + query_string
    if method == "GET":
        body = None
    request = Request(method, url, headers=headers, content=body)
    async with AsyncClient(
        timeout=88  # Firefox times out after 90 seconds, so we need to return before that.
    ) as client:
        response = await client.send(request)
    return response


def get_headers(token):
    return {
        "Authorization": f"Bearer {token}",
        "User-Agent": "Fidus Writer",
        "Content-Type": "application/json",
    }


async def get_repo(id, user):
    social_token = SocialToken.objects.get(
        account__user=user, account__provider="gitlab"
    )
    headers = get_headers(social_token.token)
    files = []
    next_url = (
        f"{GITLAB_BASE_URL}projects/{id}/repository/tree"
        "?recursive=true&per_page=4&pagination=keyset"
    )
    while next_url:
        request = Request("GET", next_url, headers=headers)
        async with AsyncClient(
            timeout=88  # Firefox times out after 90 seconds, so we need to return before that.
        ) as client:
            response = await client.send(request)
        files += json.loads(response.text)
        next_url = False
        for link_info in response.headers["Link"].split(", "):
            link, rel = link_info.split("; ")
            if rel == 'rel="next"':
                next_url = link[1:-1]
    return files


def gitlabrepo2repodata(gitlab_repo):
    return {
        "type": "gitlab",
        "name": gitlab_repo["path_with_namespace"],
        "id": gitlab_repo["id"],
        "branch": gitlab_repo["default_branch"],
    }


async def get_repos(gitlab_token):
    # TODO: API documentation unclear on whether pagination is required.
    headers = get_headers(gitlab_token)
    repos = []
    url = f"{GITLAB_BASE_URL}projects?min_access_level=30&simple=true"
    request = Request("GET", url, headers)
    async with AsyncClient(
        timeout=88  # Firefox times out after 90 seconds, so we need to return before that.
    ) as client:
        response = await client.send(request)
    content = json.loads(response.text)
    repos += map(gitlabrepo2repodata, content)
    return repos



# Generated by Django 4.1.2 on 2023-02-16 17:31

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("website", "0012_editor_group"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Editor",
        ),
    ]



# Generated by Django 4.1.2 on 2023-02-16 12:05

# Replace Editor model with a permissions group for editors

from django.db import migrations
from django.contrib.auth.management import create_permissions
from django.db.models import Count


def convert_editors(apps, schema_editor):
    # Ensure that permissions exist
    # See https://code.djangoproject.com/ticket/23422#comment:29
    for app_config in apps.get_app_configs():
        app_config.models_module = True
        create_permissions(app_config, apps=apps, verbosity=0)
        app_config.models_module = None
    Permission = apps.get_model("auth", "Permission")
    Group = apps.get_model("auth", "Group")
    ContentType = apps.get_model("contenttypes", "ContentType")
    website_ct = ContentType.objects.get(
        app_label="website", model="publication"
    )
    add_perm, _created = Permission.objects.get_or_create(
        content_type=website_ct, codename="add_publication"
    )
    change_perm, _created = Permission.objects.get_or_create(
        content_type=website_ct, codename="change_publication"
    )
    website_editor_group = (
        Group.objects.annotate(perm_count=Count("permissions"))
        .filter(perm_count=2)
        .filter(permissions=add_perm)
        .filter(permissions=change_perm)
        .first()
    )
    if not website_editor_group:
        website_editor_group = Group.objects.create(name="Website Editors")
        website_editor_group.permissions.add(add_perm)
        website_editor_group.permissions.add(change_perm)
    Editor = apps.get_model("website", "Editor")
    editors = Editor.objects.all().iterator()
    for editor in editors:
        editor.user.groups.add(website_editor_group)


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("website", "0011_alter_authors"),
    ]

    operations = [
        migrations.RunPython(convert_editors),
    ]



# Generated by Django 4.1.2 on 2023-02-16 11:12

from django.db import migrations


def convert_author(author_string):
    author_string = author_string.strip()
    if " " in author_string:
        author_parts = author_string.split(" ")
        firstname = author_parts.pop(0)
        lastname = " ".join(author_parts)
        return {"firstname": firstname, "lastname": lastname}
    else:
        return {"firstname": author_string}


def update_publications(apps, schema_editor):
    Publication = apps.get_model("website", "Publication")
    publications = Publication.objects.all().iterator()
    for publication in publications:
        publication.authors = list(map(convert_author, publication.authors))
        publication.save()


class Migration(migrations.Migration):
    dependencies = [
        ("website", "0010_alter_publication_status"),
    ]

    operations = [
        migrations.RunPython(update_publications),
    ]



from django.urls import re_path

from . import views

urlpatterns = [
    re_path("^get_book_repos/$", views.get_book_repos, name="get_book_repos"),
    re_path(
        "^update_book_repo/$", views.update_book_repo, name="update_book_repo"
    ),
    re_path(
        "^get_git_repos/reload/$",
        views.get_git_repos,
        {"reload": True},
        name="get_git_repos_reload",
    ),
    re_path("^get_git_repos/$", views.get_git_repos, name="get_git_repos"),
    re_path(
        "^get_gitlab_repo/(?P<id>.*)/$",
        views.get_gitlab_repo,
        name="get_gitlab_repo",
    ),
    re_path(
        "^proxy_github/(?P<path>.*)$", views.proxy_github, name="proxy_github"
    ),
    re_path(
        "^proxy_gitlab/(?P<path>.*)$", views.proxy_gitlab, name="proxy_gitlab"
    ),
]



# Generated by Django 4.1.2 on 2023-02-15 10:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("website", "0009_design"),
    ]

    operations = [
        migrations.AlterField(
            model_name="publication",
            name="status",
            field=models.CharField(
                choices=[
                    ("unsubmitted", "Unsubmitted"),
                    ("submitted", "Submitted"),
                    ("published", "Published"),
                    ("rejected", "Rejected"),
                    ("resubmitted", "Resubmitted"),
                ],
                default="unsubmitted",
                max_length=11,
            ),
        ),
    ]



import re
import json

from httpx import AsyncClient, Request
from allauth.socialaccount.models import SocialToken

ALLOWED_PATHS = [
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/contents/"),
    re.compile(r"^user/repos$"),
    re.compile(r"^user/repos/reload$"),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/blobs/([\w\d]+)$"),
    re.compile(
        r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/refs/heads/([\w\d]+)$"
    ),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/blobs$"),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)$"),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/commits$"),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/trees$"),
]


async def proxy(path, user, query_string, body, method):
    if not any(regex.match(path) for regex in ALLOWED_PATHS):
        raise Exception("Path not permitted.")
    social_token = SocialToken.objects.get(
        account__user=user, account__provider="github"
    )
    headers = get_headers(social_token.token)
    url = f"https://api.github.com/{path}"
    if query_string:
        url += "?" + query_string
    if method == "GET":
        body = None
    request = Request(method, url, headers=headers, content=body)
    async with AsyncClient(
        timeout=88  # Firefox times out after 90 seconds, so we need to return before that.
    ) as client:
        response = await client.send(request)
    return response


def get_headers(token):
    return {
        "Authorization": f"token {token}",
        "User-Agent": "Fidus Writer",
        "Accept": "application/vnd.github.v3+json",
    }


def githubrepo2repodata(github_repo):
    return {
        "type": "github",
        "name": github_repo["full_name"],
        "id": github_repo["id"],
        "branch": github_repo["default_branch"],
    }


async def get_repos(github_token):
    headers = get_headers(github_token)
    repos = []
    page = 1
    last_page = False
    while not last_page:
        url = f"https://api.github.com/user/repos?page={page}&per_page=100"
        request = Request("GET", url, headers=headers)
        async with AsyncClient(
            timeout=88  # Firefox times out after 90 seconds, so we need to return before that.
        ) as client:
            response = await client.send(request)
        content = json.loads(response.text)
        repos += map(githubrepo2repodata, content)
        if len(content) == 100:
            page += 1
        else:
            last_page = True
    return repos
